@@ -7,7 +7,7 @@ use Carp;
 use Dancer::Plugin;
 use Dancer qw(:syntax);
 
-our $VERSION = '0.30';
+our $VERSION = '0.40';
 
 my $settings = plugin_setting;
 
@@ -17,21 +17,6 @@ my $logoutpage = $settings->{logout_page} || '/logout';
 my $deniedpage = $settings->{denied_page} || '/login/denied';
 my $exitpage = $settings->{exit_page};
 
-if(!$settings->{no_api_change_warning}) {
-Dancer::Logger::warning(<<CHANGEWARNING);
-
-***************************************************************************
-* IMPORTANT NOTE: Dancer::Plugin::Auth::Extensible $VERSION                   *
-*                                                                         *
-* This version of DPAE contains backwards-incompatible API changes,       *
-* replacing the subroutine-attributes based approach of previous versions *
-* with new keywords.                                                      *
-*                                                                         *
-* Please see http://advent.perldancer.org/2012/16 for details.            *
-***************************************************************************
-
-CHANGEWARNING
-}
 
 =head1 NAME
 
@@ -46,9 +31,11 @@ provides role-based access control, and supports various authentication
 methods/sources (config file, database, Unix system users, etc).
 
 Designed to support multiple authentication realms and to be as extensible as
-possible, and to make secure password handling easy (the base class for auth
+possible, and to make secure password handling easy.  The base class for auth
 providers makes handling C<RFC2307>-style hashed passwords really simple, so you
-have no excuse for storing plain-text passwords).
+have no excuse for storing plain-text passwords.  A simple script to generate
+RFC2307-style hashed passwords is included, or you can use L<Crypt::SaltedHash>
+yourself to do so, or use the C<slappasswd> utility if you have it installed.
 
 
 =head1 SYNOPSIS
@@ -123,7 +110,7 @@ Keywords are provided to check if a user is logged in / has appropriate roles.
 
 If the user is not logged in, they will be redirected to the login page URL to
 log in.  The default URL is C</login> - this may be changed with the
-C<login_url> option.
+C<login_page> option.
 
 =item require_role - require the user to have a specified role
 
@@ -161,6 +148,22 @@ redirected to the access denied URL.
 By default, the plugin adds a route to present a simple login form at that URL.
 If you would rather add your own, set the C<no_default_pages> setting to a true
 value, and define your own route which responds to C</login> with a login page.
+Alternatively you can let DPAE add the routes and handle the status codes, etc.
+and simply define the setting C<login_page_handler> and/or
+C<permission_denied_page_handler> with the name of a subroutine to be called to
+handle the route. Note that it must be a fully qualified sub. E.g.
+
+    plugins:
+      Auth::Extensible:
+        login_page_handler: 'My::App:login_page_handler'
+        permission_denied_page_handler: 'My::App:permission_denied_page_handler'
+
+Then in your code you might simply use a template:
+
+    sub permission_denied_page_handler {
+        template 'account/login';
+    }
+
 
 If the user is logged in, but tries to access a route which requires a specific
 role they don't have, they will be redirected to the "permission denied" page
@@ -319,6 +322,8 @@ sub _build_wrapper {
         }
 
         if ($role_match) {
+            # We're happy with their roles, so go head and execute the route
+            # handler coderef.
             return $coderef->();
         }
 
@@ -514,8 +519,11 @@ sub auth_provider {
     if ($provider_class !~ /::/) {
         $provider_class = __PACKAGE__ . "::Provider::$provider_class";
     }
-    Dancer::ModuleLoader->load($provider_class)
-        or die "Cannot load provider $provider_class";
+    my ($ok, $error) = Dancer::ModuleLoader->load($provider_class);
+
+    if (! $ok) {
+        die "Cannot load provider $provider_class: $error";
+    }
 
     return $realm_provider{$realm} = $provider_class->new($realm_settings);
 }
@@ -546,14 +554,25 @@ sub _try_realms {
 }
 
 # Set up routes to serve default pages, if desired
-if (!$settings->{no_default_pages}) {
+if ( !$settings->{no_default_pages} ) {
     get $loginpage => sub {
+        if(logged_in_user()) {
+            redirect params->{return_url} || $userhomepage;
+        }
+
         status 401;
-        return _default_login_page();
+        my $_default_login_page =
+          $settings->{login_page_handler} || '_default_login_page';
+        no strict 'refs';
+        return &{$_default_login_page}();
     };
     get $deniedpage => sub {
         status 403;
-        return _default_permission_denied_page();
+        my $_default_permission_denied_page =
+          $settings->{permission_denied_page_handler}
+          || '_default_permission_denied_page';
+        no strict 'refs';
+        return &{$_default_permission_denied_page}();
     };
 }
 
@@ -563,11 +582,33 @@ if (!$settings->{no_login_handler}) {
 
 # Handle logging in...
 post $loginpage => sub {
+    # For security, ensure the username and password are straight scalars; if
+    # the app is using a serializer and we were sent a blob of JSON, they could
+    # have come from that JSON, and thus could be hashrefs (JSON SQL injection)
+    # - for database providers, feeding a carefully crafted hashref to the SQL
+    # builder could result in different SQL to what we'd expect.
+    # For instance, if we pass password => params->{password} to an SQL builder,
+    # we'd expect the query to include e.g. "WHERE password = '...'" (likely
+    # with paremeterisation) - but if params->{password} was something
+    # different, e.g. { 'like' => '%' }, we might end up with some SQL like
+    # WHERE password LIKE '%' instead - which would not be a Good Thing.
+    my ($username, $password) = @{ params() }{qw(username password)};
+    for ($username, $password) {
+        if (ref $_) {
+            # TODO: handle more cleanly
+            die "Attempt to pass a reference as username/password blocked";
+        }
+    }
+
+    if(logged_in_user()) {
+        redirect params->{return_url} || $userhomepage;
+    }
+
     my ($success, $realm) = authenticate_user(
-        params->{username}, params->{password}
+        $username, $password
     );
     if ($success) {
-        session logged_in_user => params->{username};
+        session logged_in_user => $username;
         session logged_in_user_realm => $realm;
         redirect params->{return_url} || $userhomepage;
     } else {
